-- Designed by Toshio Iwata at DIGITALFILTER.COM 2013/03/19

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEOUT_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
--		SoftReset_N	: IN std_logic;
		ViewMode : in std_logic_vector(3 downto 0);
--		BigLineN : in std_logic_vector(8 downto 0);
--		RD_ADDR_LOAD : in std_logic;
--		RD_ADDR_ENB : in std_logic;
--		WR_ADDR_CLR : in std_logic;
		PIXSTB: in std_logic;
		PIXWR : in std_logic;
		PIXRD : in std_logic;
  RD_BUF_SEL : out std_logic_vector(3 downto 0);
  WR_BUF_SEL : out std_logic_vector(1 downto 0);
  WR_BUF_SEL2 : out std_logic_vector(1 downto 0);
  WR_BUF_N : out std_logic;
  RD_BUF_ADDR : out std_logic_vector(7 downto 0);
  SsdLineCount : out std_logic_vector(8 downto 0);
  SsdLineCount2 : out std_logic_vector(8 downto 0);
  SsdPixCount : out std_logic_vector(8 downto 0);
  SsdPixCount2 : out std_logic_vector(8 downto 0);
  OutChSel : out std_logic_vector(6 downto 0);
  CO9 : out std_logic_vector(3 downto 0);
  CO512 : out std_logic_vector(8 downto 0);
  CO512_dly2 : out std_logic_vector(8 downto 0);
  PIX2LOGIC : in std_logic_vector(15 downto 0);
LBRDY : out std_logic;
CamHsync_edge : in std_logic;
VgaHsync_edge : in std_logic;
DISPJUDGE : in std_logic;
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		CamLineCount : in std_logic_vector(8 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic );
END LINEOUT_CTRL;

ARCHITECTURE structure OF LINEOUT_CTRL IS

signal ODDLINE, oddline_dly1, oddline_dly2, oddline_dly3, oddline_edge : std_logic;
 signal ls2b : std_logic_vector(1 downto 0);
 signal LB_ADDR : std_logic_vector(7 downto 0);
 signal baseaddr : std_logic_vector(7 downto 0);
signal PIXRD_edge, PIXRD_dly1, PIXRD_dly2 : std_logic;
signal PIXWR_edge, PIXWR_dly1, PIXWR_dly2 : std_logic;
signal WaitLine : std_logic_vector(2 downto 0);
signal  pixstb_count : std_logic_vector(9 downto 0);
signal  pixstb_count_plus : std_logic_vector(9 downto 0);
signal LBRDY_sig : std_logic;
	signal ViewMode_dly1, ViewMode_dly2 : std_logic_vector(3 downto 0);
  signal ViewMode5_edge : std_logic;
  signal ViewMode6_edge : std_logic;
 signal count4wraddr : std_logic_vector(9 downto 0);
signal SsdStart : std_logic;
signal count512 : std_logic_vector(8 downto 0); -- counts input channels(+1)
signal count512_clr : std_logic;
signal count512_dly1, count512_dly2 : std_logic_vector(8 downto 0);
signal count3 : std_logic_vector(7 downto 0); -- counts horizontal 3 pixs
signal count9 : std_logic_vector(3 downto 0); -- counts 3x3 pixs
signal count9_clr : std_logic;
signal count9_dly1, count9_dly2 : std_logic_vector(3 downto 0);
signal count600 : std_logic_vector(13 downto 0); -- counts 300 areas
signal count600_clr : std_logic;
--signal count600_dly1, count600_dly2 : std_logic_vector(8 downto 0);
signal count10_64_dly1, count10_64_dly2 : std_logic_vector(6 downto 0);
signal count10 : std_logic_vector(6 downto 0); -- counts 300 areas
signal countV : std_logic_vector(8 downto 0);
signal rdbufsel : std_logic_vector(3 downto 0);
signal rdbufsel_dly1, rdbufsel_dly2 : std_logic_vector(3 downto 0);
signal loadval : std_logic_vector(1 downto 0);
signal WR_ADDR_CLR_dly1, WR_ADDR_CLR_dly2, WR_ADDR_CLR_edge : std_logic;
signal RD_ADDR_ENB_dly1, RD_ADDR_ENB_dly2, RD_ADDR_ENB_edge : std_logic;
signal WRN2DL_sig, DISPJUDGE_latch : std_logic;
signal countV_dly1, countV_dly2 : std_logic_vector(8 downto 0);
signal SsdLineCount_tmp, SsdLineCount_dly1 : std_logic_vector(8 downto 0);

BEGIN
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     oddline_dly3 <= '0';
     oddline_dly2 <= '0';
     oddline_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     oddline_dly3 <= oddline_dly2;
     oddline_dly2 <= oddline_dly1;
     oddline_dly1 <= ODDLINE;
   end if;  
end process;

  ODDLINE <= VgaLineCount(1);
  oddline_edge <= '1' when (oddline_dly1 = '0' and oddline_dly2 = '1') else '0';

  WR_BUF_N <= '0' when count512_clr = '1' and count600(0) = '1' else '1';
  --WR_BUF_SEL <= countV_dly8(1 downto 0); -- for lay5 2022/05/25
  --WR_BUF_SEL2 <= countV_dly8(2 downto 1); -- because of pooling -- for lay4, needs more dly
  WR_BUF_SEL <= countV_dly1(1 downto 0); -- for lay14
  WR_BUF_SEL2 <= countV_dly2(1 downto 0); 

-------------------------------------------------------------- 
  WRN2DL_sig <= '0' when (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     else '1';
                       
-------------------------------------------------------------------
  ls2b <= "00" when winp0 = '1' else
          "01" when winp1 = '1' else
          "10" when winp2 = '1' else
          "11";

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
	if( RST_N = '0' ) then
		LB_ADDR <= (others => '0');
	elsif( CLK100M'event and CLK100M = '1' ) then
		--LB_ADDR <= baseaddr + count3; -- for lay11
		LB_ADDR <= baseaddr; -- for lay14
	end if;  
end process;
          
--baseaddr <= ("0" & count600(10 downto 4)); -- for lay11
baseaddr <= ("00" & count600(12 downto 7)); -- for lay11

OutChSel <= count600(6 downto 0); -- for lay11 

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(SsdStart = '1' ) then
         countV <= (others => '0');
       elsif(count600_clr = '1' and countV /= 63) then 
         countV <= countV + 1;
       end if;
   end if;
end process;


-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     countV_dly2 <= (others => '0');
     countV_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count512_clr = '1') then
          if( count600(3 downto 0) = "1111" ) then -- 2022/10/02
             countV_dly2 <= countV_dly1; -- for writing Relu-out
             countV_dly1 <= countV; -- for writing Relu-out
          end if;
       end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then --or SoftReset_N = '0' ) then
     count600 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
       if(count600_clr = '1' or SsdStart = '1' ) then
         count600 <= (others => '0');
       elsif(count512_clr = '1') then
         count600 <= count600 + 1;
       end if;
   end if;
end process;

--count600_clr <= '1' when count600 = 2699 and count512 = 1023 else '0'; -- 10ns x 1024 x 2700 = 27.648ms -- for lay15 count9 is not needed
count600_clr <= '1' when count600 = 10799 and count512 = 511 else '0'; -- 10ns x 512 x 10800 = 55.296ms -- for ex3

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count512 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count512_clr = '1' or SsdStart = '1' ) then
       count512 <= (others => '0');
     elsif(count9_clr = '1') then
       count512 <= count512 + 1;
     end if;
  end if;
end process;

--count512_clr <= '1' when count512 = 1023 else '0'; -- for lay15 count9 is not needed
count512_clr <= '1' when count512 = 511 else '0'; -- for lay15 count9 is not needed

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then -- or SoftReset_N = '0' ) then
     count9 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(count9_clr = '1' or  SsdStart = '1' ) then 
       count9 <= (others => '0');
     else
       count9 <= count9 + 1;
     end if;
  end if;
end process;

--  count9_clr <= '1' when count9 = 8 else '0';
  count9_clr <= '1'; -- for lay15 because kernel is 1x1

  count3 <= "00000000" when count9 = 0 or count9 = 3 or  count9 = 6 else 
		"00000001" when count9 = 1 or count9 = 4 or  count9 = 7 else 
		"00000010";

  rdbufsel <= countV(3 downto 0); 

  RD_BUF_SEL <= rdbufsel_dly2;

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    DISPJUDGE_latch <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(VgaHsync_edge = '1') then
      DISPJUDGE_latch <= DISPJUDGE;
    end if;
  end if;
end process;

  SsdStart <= '1' when VgaHsync_edge = '1' and DISPJUDGE_latch = '0' else '0';

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     rdbufsel_dly2 <= (others => '0');
     rdbufsel_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     rdbufsel_dly2 <= rdbufsel_dly1;
     rdbufsel_dly1 <= rdbufsel;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count9_dly2 <= (others => '0');
     count9_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count9_dly2 <= count9_dly1;
     count9_dly1 <= count9;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count512_dly2 <= (others => '0');
     count512_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     count512_dly2 <= count512_dly1;
     count512_dly1 <= count512;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     SsdLineCount <= (others => '0');
     SsdLineCount_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     SsdLineCount <= SsdLineCount_dly1; -- delay 20ns to make it the same timing with ram_x1_rd_sel 2022/04/17
     SsdLineCount_dly1 <= SsdLineCount_tmp;
   end if;
end process;

--  count10 <= count600(10 downto 4); -- for lay11; 
  count10 <= count600(13 downto 7); -- for ex3; 

  SsdLineCount_tmp <= countV;
  SsdLineCount2 <= countV_dly2; -- for lay9, needs more dly
  --SsdLineCount2 <= countV_dly1; -- for lay8, needs more dly
  --SsdLineCount2 <= countV_dly2; -- for lay6, needs more dly for pooling
  SsdPixCount <= "00" & count10; 
  SsdPixCount2 <= "00" & count10_64_dly2; -- for lay6 from (9 downto 1); -- for layer2 this would be better (not tricky)
  CO9 <= count9;
  CO512 <= count512;
  CO512_dly2 <= count512_dly2;

process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count10_64_dly2 <= (others => '0');
     count10_64_dly1 <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
      if(count512_clr = '1') then
         if( count600(3 downto 0) = "1111" ) then -- 2022/10/02
            count10_64_dly2 <= count10_64_dly1;
            count10_64_dly1 <= count10;
         end if;
      end if;
   end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count4wraddr <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(VgaHsync_edge = '1') then 
       count4wraddr <= (others => '0');
     elsif(WRN2DL_sig = '0' and count4wraddr /= 607) then -- 1023) then
       count4wraddr <= count4wraddr + 1;
     end if;
  end if;
end process;

-------------------------------------------------------------- 
  RD_BUF_ADDR <= LB_ADDR + 1; -- for lay15 no padding

-------------------------------------------------------------- 
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXRD_dly2 <= '0';
      PIXRD_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXRD_dly2 <= PIXRD_dly1;
    PIXRD_dly1 <= PIXRD;
  end if;
end process;

  PIXRD_edge <= '1' when PIXRD_dly1 = '0' and PIXRD_dly2 = '1' else '0';
  
-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0') then
      PIXWR_dly2 <= '0';
      PIXWR_dly1 <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PIXWR_dly2 <= PIXWR_dly1;
    PIXWR_dly1 <= PIXWR;
  end if;
end process;

  PIXWR_edge <= '1' when PIXWR_dly1 = '0' and PIXWR_dly2 = '1' else '0';

-------------------------------------------------------------------
process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    WaitLine <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(PIXWR_edge = '1' or PIXRD_edge = '1') then
      WaitLine <= (others => '0');
    elsif(CamHsync_edge = '1') then
      if(WaitLine /= 3) then -- 2012/11/15
        WaitLine <= WaitLine + 1;
      end if;
    end if;
  end if;
end process;

  LBRDY_sig <= '1' when not(WaitLine = 1 or WaitLine = 2) and (ViewMode = "0101" or ViewMode = "0110") else '0'; -- 2012/11/15
  LBRDY <= LBRDY_sig;
  
-------------------------------------------------------------- 
process( RST_N, PIXSTB, PIXRD_edge, PIXWR_edge )  
begin 
   if( RST_N = '0' or PIXRD_edge = '1'  or PIXWR_edge = '1' ) then
     pixstb_count <= (others => '0');
   elsif( PIXSTB'event and PIXSTB = '1' ) then
     pixstb_count <= pixstb_count + 1;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     pixstb_count_plus <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     pixstb_count_plus <= pixstb_count + 96;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     ViewMode_dly2 <= (others => '0');
     ViewMode_dly1 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     ViewMode_dly2 <= ViewMode_dly1;
     ViewMode_dly1 <= ViewMode;
   end if;  
end process;
   
-------------------------------------------------------------- 
  ViewMode5_edge <= '1' when ViewMode_dly1 = "0101" and ViewMode_dly2 /= "0101" else '0';
  ViewMode6_edge <= '1' when ViewMode_dly1 = "0110" and ViewMode_dly2 /= "0110" else '0';

END structure;
